#include "main.h"
#include "monetra.h"

#define if_chkvar(var) if ((var != NULL) && strlen(var))

M_uintptr M_SPEC M_Ping(M_CONN *conn)
{
	M_uintptr identifier;

	identifier = M_TransNew(conn);
	M_TransKeyVal(conn, identifier, "action", "ping");
	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

#if !defined(LIBMONETRA_SPEC_STDCALL) && defined(M_DEPRECATED)
/* TRANSACTION ROUTINES */
M_uintptr M_SPEC M_Sale(
	M_CONN *conn,
	char *username,
	char *password,
	char *trackdata,
	char *account,
	char *expdate,
	double amount,
	char *street,
	char *zip,
	char *cv,
	char *comments,
	char *clerkid,
	char *stationid,
	long ptrannum
) {
	M_uintptr identifier;

	if (
		(username == NULL) ||
		(password == NULL) ||
		((trackdata == NULL) && ((account == NULL) || (expdate == NULL))) ||
		(amount <= 0)
	) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_SALE);
	M_TransParam(conn, identifier, MC_USERNAME, username);
	M_TransParam(conn, identifier, MC_PASSWORD, password);
	if_chkvar(trackdata) M_TransParam(conn, identifier, MC_TRACKDATA, trackdata);
	if_chkvar(account) M_TransParam(conn, identifier, MC_ACCOUNT, account);
	if_chkvar(expdate) M_TransParam(conn, identifier, MC_EXPDATE, expdate);
	if_chkvar(street) M_TransParam(conn, identifier, MC_STREET, street);
	if_chkvar(zip) M_TransParam(conn, identifier, MC_ZIP, zip);
	if_chkvar(cv) M_TransParam(conn, identifier, MC_CV, cv);
	if (amount > 0.00) {
		M_TransParam(conn, identifier, MC_AMOUNT, amount);
	}

	if_chkvar(comments) M_TransParam(conn, identifier, MC_COMMENTS, comments);
	if_chkvar(clerkid) M_TransParam(conn, identifier, MC_CLERKID, clerkid);
	if_chkvar(stationid) M_TransParam(conn, identifier, MC_STATIONID, stationid);
	if (ptrannum >= 0) {
		M_TransParam(conn, identifier, MC_PTRANNUM, ptrannum);
	}

	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

M_uintptr M_SPEC M_PreAuth(
	M_CONN *conn,
	char *username,
	char *password,
	char *trackdata,
	char *account,
	char *expdate,
	double amount,
	char *street,
	char *zip,
	char *cv,
	char *comments,
	char *clerkid,
	char *stationid,
	long ptrannum
) {
	M_uintptr identifier;

	if (
		(username == NULL) ||
		(password == NULL) ||
		((trackdata == NULL) && ((account == NULL) || (expdate == NULL))) ||
		(amount <= 0)
	) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_PREAUTH);
	M_TransParam(conn, identifier, MC_USERNAME, username);
	M_TransParam(conn, identifier, MC_PASSWORD, password);
	if_chkvar(trackdata) M_TransParam(conn, identifier, MC_TRACKDATA, trackdata);
	if_chkvar(account) M_TransParam(conn, identifier, MC_ACCOUNT, account);
	if_chkvar(expdate) M_TransParam(conn, identifier, MC_EXPDATE, expdate);
	if_chkvar(street) M_TransParam(conn, identifier, MC_STREET, street);
	if_chkvar(zip) M_TransParam(conn, identifier, MC_ZIP, zip);
	if_chkvar(cv) M_TransParam(conn, identifier, MC_CV, cv);
	if (amount > 0.00) {
		M_TransParam(conn, identifier, MC_AMOUNT, amount);
	}

	if_chkvar(comments) M_TransParam(conn, identifier, MC_COMMENTS, comments);
	if_chkvar(clerkid) M_TransParam(conn, identifier, MC_CLERKID, clerkid);
	if_chkvar(stationid) M_TransParam(conn, identifier, MC_STATIONID, stationid);
	if (ptrannum >= 0) {
		M_TransParam(conn, identifier, MC_PTRANNUM, ptrannum);
	}

	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

M_uintptr M_SPEC M_Void(M_CONN *conn, char *username, char *password, M_int64 ttid, long ptrannum)
{
	M_uintptr identifier;

	if (
		(username == NULL) ||
		(password == NULL) ||
		((ttid == -1) && (ptrannum == -1))
	) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_VOID);
	M_TransParam(conn, identifier, MC_USERNAME, username);
	M_TransParam(conn, identifier, MC_PASSWORD, password);

	if (ttid > 0) {
		M_TransParam(conn, identifier, MC_TTID, ttid);
	} else if (ptrannum >= 0) {
		M_TransParam(conn, identifier, MC_PTRANNUM, ptrannum);
	}

	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

M_uintptr M_SPEC M_PreAuthCompletion(M_CONN *conn, char *username, char *password,
                                     double finalamount, M_int64 ttid, long ptrannum)
{
	M_uintptr identifier;

	if (
		(username == NULL) ||
		(password == NULL) ||
		(finalamount <= 0) ||
		((ttid == -1) && (ptrannum == -1))
	) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_PREAUTHCOMPLETE);
	M_TransParam(conn, identifier, MC_USERNAME, username);
	M_TransParam(conn, identifier, MC_PASSWORD, password);
	M_TransParam(conn, identifier, MC_AMOUNT, finalamount);
	if (ttid > 0) {
		M_TransParam(conn, identifier, MC_TTID, ttid);
	} else if (ptrannum >= 0) {
		M_TransParam(conn, identifier, MC_PTRANNUM, ptrannum);
	}

	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

M_uintptr M_SPEC M_Force(
	M_CONN *conn,
	char *username,
	char *password,
	char *trackdata,
	char *account,
	char *expdate,
	double amount,
	char *authcode,
	char *comments,
	char *clerkid,
	char *stationid,
	long ptrannum
) {
	M_uintptr identifier;

	if (
		(username == NULL) ||
		(password == NULL) ||
		((trackdata == NULL) && ((account == NULL) || (expdate == NULL))) ||
		(amount <= 0)
	) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_FORCE);
	M_TransParam(conn, identifier, MC_USERNAME, username);
	M_TransParam(conn, identifier, MC_PASSWORD, password);
	if_chkvar(trackdata) M_TransParam(conn, identifier, MC_TRACKDATA, trackdata);
	if_chkvar(account) M_TransParam(conn, identifier, MC_ACCOUNT, account);
	if_chkvar(expdate) M_TransParam(conn, identifier, MC_EXPDATE, expdate);
	if (amount > 0.00) {
		M_TransParam(conn, identifier, MC_AMOUNT, amount);
	}

	if_chkvar(authcode) M_TransParam(conn, identifier, MC_APPRCODE, authcode);
	if_chkvar(comments) M_TransParam(conn, identifier, MC_COMMENTS, comments);
	if_chkvar(clerkid) M_TransParam(conn, identifier, MC_CLERKID, clerkid);
	if_chkvar(stationid) M_TransParam(conn, identifier, MC_STATIONID, stationid);
	if (ptrannum >= 0) {
		M_TransParam(conn, identifier, MC_PTRANNUM, ptrannum);
	}

	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

M_uintptr M_SPEC M_Return(
	M_CONN *conn,
	char *username,
	char *password,
	char *trackdata,
	char *account,
	char *expdate,
	double amount,
	char *comments,
	char *clerkid,
	char *stationid,
	long ptrannum
) {
	M_uintptr identifier;

	if (
		(username == NULL) ||
		(password == NULL) ||
		((trackdata == NULL) && ((account == NULL) || (expdate == NULL))) ||
		(amount <= 0)
	) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_RETURN);
	M_TransParam(conn, identifier, MC_USERNAME, username);
	M_TransParam(conn, identifier, MC_PASSWORD, password);
	if_chkvar(trackdata) M_TransParam(conn, identifier, MC_TRACKDATA, trackdata);
	if_chkvar(account) M_TransParam(conn, identifier, MC_ACCOUNT, account);
	if_chkvar(expdate) M_TransParam(conn, identifier, MC_EXPDATE, expdate);
	if (amount > 0.00) {
		M_TransParam(conn, identifier, MC_AMOUNT, amount);
	}

	if_chkvar(comments) M_TransParam(conn, identifier, MC_COMMENTS, comments);
	if_chkvar(clerkid) M_TransParam(conn, identifier, MC_CLERKID, clerkid);
	if_chkvar(stationid) M_TransParam(conn, identifier, MC_STATIONID, stationid);
	if (ptrannum >= 0) {
		M_TransParam(conn, identifier, MC_PTRANNUM, ptrannum);
	}

	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

M_uintptr M_SPEC M_Override(
	M_CONN *conn,
	char *username,
	char *password,
	char *trackdata,
	char *account,
	char *expdate,
	double amount,
	char *street,
	char *zip,
	char *cv,
	char *comments,
	char *clerkid,
	char *stationid,
	long ptrannum
) {
	M_uintptr identifier;

	if (
		(username == NULL) ||
		(password == NULL) ||
		((trackdata == NULL) && ((account == NULL) || (expdate == NULL))) ||
		(amount <= 0)
	) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_OVERRIDE);
	M_TransParam(conn, identifier, MC_USERNAME, username);
	M_TransParam(conn, identifier, MC_PASSWORD, password);
	if_chkvar(trackdata) M_TransParam(conn, identifier, MC_TRACKDATA, trackdata);
	if_chkvar(account) M_TransParam(conn, identifier, MC_ACCOUNT, account);
	if_chkvar(expdate) M_TransParam(conn, identifier, MC_EXPDATE, expdate);
	if_chkvar(street) M_TransParam(conn, identifier, MC_STREET, street);
	if_chkvar(zip) M_TransParam(conn, identifier, MC_ZIP, zip);
	if_chkvar(cv) M_TransParam(conn, identifier, MC_CV, cv);
	if (amount > 0.00) {
		M_TransParam(conn, identifier, MC_AMOUNT, amount);
	}

	if_chkvar(comments) M_TransParam(conn, identifier, MC_COMMENTS, comments);
	if_chkvar(clerkid) M_TransParam(conn, identifier, MC_CLERKID, clerkid);
	if_chkvar(stationid) M_TransParam(conn, identifier, MC_STATIONID, stationid);
	if (ptrannum >= 0) {
		M_TransParam(conn, identifier, MC_PTRANNUM, ptrannum);
	}

	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

M_uintptr M_SPEC M_Settle(M_CONN *conn, char *username, char *password, char *batch)
{
	M_uintptr identifier;

	if ((username == NULL) || (password == NULL)) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_SETTLE);
	M_TransParam(conn, identifier, MC_USERNAME, username);
	M_TransParam(conn, identifier, MC_PASSWORD, password);
	if ((batch == NULL) || *batch == '\0') {
		M_TransParam(conn, identifier, MC_BATCH, "all");
	} else {
		M_TransParam(conn, identifier, MC_BATCH, batch);
	}

	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

/* USER-LEVEL ADMINISTRATIVE REQUESTS */

M_uintptr M_SPEC M_Gut(
	M_CONN *conn,
	char *username,
	char *password,
	int type,
	char *myacct,
	char *clerkid,
	char *stationid,
	char *comments,
	double ptrannum,
	char *bdate,
	char *edate
) {
	M_uintptr identifier;

	if ((username == NULL) || (password == NULL)) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_ADMIN);
	M_TransParam(conn, identifier, MC_ADMIN, MC_ADMIN_GUT);
	M_TransParam(conn, identifier, MC_USERNAME, username);
	M_TransParam(conn, identifier, MC_PASSWORD, password);
	if (type > 0) {
		M_TransParam(conn, identifier, MC_AUDITTYPE, type);
	}

	if_chkvar(myacct) M_TransParam(conn, identifier, MC_ACCT, myacct);
	if_chkvar(clerkid) M_TransParam(conn, identifier, MC_CLERKID, clerkid);
	if_chkvar(stationid) M_TransParam(conn, identifier, MC_STATIONID, stationid);
	if_chkvar(comments) M_TransParam(conn, identifier, MC_COMMENTS, comments);
	if_chkvar(bdate) M_TransParam(conn, identifier, MC_BDATE, bdate);
	if_chkvar(edate) M_TransParam(conn, identifier, MC_EDATE, edate);
	if (ptrannum >= 0) {
		M_TransParam(conn, identifier, MC_PTRANNUM, ptrannum);
	}

	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

M_uintptr M_SPEC M_Gl(
	M_CONN *conn,
	char *username,
	char *password,
	int type,
	char *myacct,
	char *batch,
	char *clerkid,
	char *stationid,
	char *comments,
	double ptrannum,
	char *bdate,
	char *edate
) {
	M_uintptr identifier;

	if ((username == NULL) || (password == NULL)) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_ADMIN);
	M_TransParam(conn, identifier, MC_ADMIN, MC_ADMIN_GL);
	M_TransParam(conn, identifier, MC_USERNAME, username);
	M_TransParam(conn, identifier, MC_PASSWORD, password);
	if (type > 0) {
		M_TransParam(conn, identifier, MC_AUDITTYPE, type);
	}

	if_chkvar(myacct) M_TransParam(conn, identifier, MC_ACCT, myacct);
	if_chkvar(batch) M_TransParam(conn, identifier, MC_BATCH, batch);
	if_chkvar(clerkid) M_TransParam(conn, identifier, MC_CLERKID, clerkid);
	if_chkvar(stationid) M_TransParam(conn, identifier, MC_STATIONID, stationid);
	if_chkvar(comments) M_TransParam(conn, identifier, MC_COMMENTS, comments);
	if_chkvar(bdate) M_TransParam(conn, identifier, MC_BDATE, bdate);
	if_chkvar(edate) M_TransParam(conn, identifier, MC_EDATE, edate);
	if (ptrannum >= 0) {
		M_TransParam(conn, identifier, MC_PTRANNUM, ptrannum);
	}

	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

M_uintptr M_SPEC M_Gft(
	M_CONN *conn,
	char *username,
	char *password,
	int type,
	char *myacct,
	char *clerkid,
	char *stationid,
	char *comments,
	double ptrannum,
	char *bdate,
	char *edate
) {
	M_uintptr identifier;

	if ((username == NULL) || (password == NULL)) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_ADMIN);
	M_TransParam(conn, identifier, MC_ADMIN, MC_ADMIN_GFT);
	M_TransParam(conn, identifier, MC_USERNAME, username);
	M_TransParam(conn, identifier, MC_PASSWORD, password);
	if (type > 0) {
		M_TransParam(conn, identifier, MC_AUDITTYPE, type);
	}

	if_chkvar(myacct) M_TransParam(conn, identifier, MC_ACCT, myacct);
	if_chkvar(clerkid) M_TransParam(conn, identifier, MC_CLERKID, clerkid);
	if_chkvar(stationid) M_TransParam(conn, identifier, MC_STATIONID, stationid);
	if_chkvar(comments) M_TransParam(conn, identifier, MC_COMMENTS, comments);
	if_chkvar(bdate) M_TransParam(conn, identifier, MC_BDATE, bdate);
	if_chkvar(edate) M_TransParam(conn, identifier, MC_EDATE, edate);
	if (ptrannum >= 0) {
		M_TransParam(conn, identifier, MC_PTRANNUM, ptrannum);
	}

	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

/*
 * long M_Ub(M_CONN *conn, char *username, char *password)
 * {
 * _M_CONN *conn=(_M_CONN *)(*conn);
 * M_TRANS trans;
 * M_ClearTransaction(&trans);
 * if (username == NULL || password == NULL)
 *  return(M_ERROR);
 * trans.trans_type=M_UB;
 * trans.username=username;
 * trans.password=password;
 * return(M_SendTransaction(conn, trans));
 * }
 */

M_uintptr M_SPEC M_Rs(M_CONN *conn, char *username, char *password, char *stationid)
{
	M_uintptr identifier;

	if ((username == NULL) || (password == NULL) || (stationid == NULL)) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_ADMIN);
	M_TransParam(conn, identifier, MC_ADMIN, MC_ADMIN_RS);
	M_TransParam(conn, identifier, MC_USERNAME, username);
	M_TransParam(conn, identifier, MC_PASSWORD, password);
	if_chkvar(stationid) M_TransParam(conn, identifier, MC_STATIONID, stationid);

	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

M_uintptr M_SPEC M_Qc(
	M_CONN *conn,
	char *username,
	char *password,
	char *clerkid,
	char *stationid,
	char *comments,
	double ptrannum
) {
	M_uintptr identifier;

	if ((username == NULL) || (password == NULL)) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_ADMIN);
	M_TransParam(conn, identifier, MC_ADMIN, MC_ADMIN_QC);
	M_TransParam(conn, identifier, MC_USERNAME, username);
	M_TransParam(conn, identifier, MC_PASSWORD, password);
	if_chkvar(clerkid) M_TransParam(conn, identifier, MC_CLERKID, clerkid);
	if_chkvar(stationid) M_TransParam(conn, identifier, MC_STATIONID, stationid);
	if_chkvar(comments) M_TransParam(conn, identifier, MC_COMMENTS, comments);
	if (ptrannum >= 0) {
		M_TransParam(conn, identifier, MC_PTRANNUM, ptrannum);
	}

	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

M_uintptr M_SPEC M_Chkpwd(M_CONN *conn, char *username, char *password)
{
	M_uintptr identifier;

	if ((username == NULL) || (password == NULL)) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_CHKPWD);
	M_TransParam(conn, identifier, MC_USERNAME, username);
	M_TransParam(conn, identifier, MC_PASSWORD, password);

	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

M_uintptr M_SPEC M_Bt(M_CONN *conn, char *username, char *password)
{
	M_uintptr identifier;

	if ((username == NULL) || (password == NULL)) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_ADMIN);
	M_TransParam(conn, identifier, MC_ADMIN, MC_ADMIN_BT);
	M_TransParam(conn, identifier, MC_USERNAME, username);
	M_TransParam(conn, identifier, MC_PASSWORD, password);

	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

/* ENGINE-ADMIN REQUESTS */

M_uintptr M_SPEC M_Chngpwd(M_CONN *conn, char *adminpassword, char *newpassword)
{
	M_uintptr identifier;

	if ((adminpassword == NULL) || (newpassword == NULL)) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_CHNGPWD);
	M_TransParam(conn, identifier, MC_USERNAME, "MCVEADMIN");
	M_TransParam(conn, identifier, MC_PASSWORD, adminpassword);
	if_chkvar(newpassword) M_TransParam(conn, identifier, MC_PWD, newpassword);
	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

M_uintptr M_SPEC M_ListUsers(M_CONN *conn, char *password)
{
	M_uintptr identifier;

	if (password == NULL) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_LISTUSERS);
	M_TransParam(conn, identifier, MC_USERNAME, "MCVEADMIN");
	M_TransParam(conn, identifier, MC_PASSWORD, password);
	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

M_uintptr M_SPEC M_EnableUser(M_CONN *conn, char *adminpwd, char *user)
{
	M_uintptr identifier;

	if ((adminpwd == NULL) || (user == NULL)) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_ENABLEUSER);
	M_TransParam(conn, identifier, MC_USERNAME, "MCVEADMIN");
	M_TransParam(conn, identifier, MC_PASSWORD, adminpwd);
	if_chkvar(user) M_TransParam(conn, identifier, MC_USER, user);
	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

M_uintptr M_SPEC M_DisableUser(M_CONN *conn, char *adminpwd, char *user)
{
	M_uintptr identifier;

	if ((adminpwd == NULL) || (user == NULL)) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_DISABLEUSER);
	M_TransParam(conn, identifier, MC_USERNAME, "MCVEADMIN");
	M_TransParam(conn, identifier, MC_PASSWORD, adminpwd);
	if_chkvar(user) M_TransParam(conn, identifier, MC_USER, user);
	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

M_uintptr M_SPEC M_GetUserInfo(M_CONN *conn, char *adminpwd, char *user)
{
	M_uintptr identifier;

	if ((adminpwd == NULL) || (user == NULL)) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_GETUSERINFO);
	M_TransParam(conn, identifier, MC_USERNAME, "MCVEADMIN");
	M_TransParam(conn, identifier, MC_PASSWORD, adminpwd);
	if_chkvar(user) M_TransParam(conn, identifier, MC_USER, user);
	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

M_uintptr M_SPEC M_DelUser(M_CONN *conn, char *password, char *user)
{
	M_uintptr identifier;

	if ((password == NULL) || (user == NULL)) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_DELUSER);
	M_TransParam(conn, identifier, MC_USERNAME, "MCVEADMIN");
	M_TransParam(conn, identifier, MC_PASSWORD, password);
	if_chkvar(user) M_TransParam(conn, identifier, MC_USER, user);
	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

M_uintptr M_SPEC M_ListStats(M_CONN *conn, char *adminpwd)
{
	M_uintptr identifier;

	if (adminpwd == NULL) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_LISTSTATS);
	M_TransParam(conn, identifier, MC_USERNAME, "MCVEADMIN");
	M_TransParam(conn, identifier, MC_PASSWORD, adminpwd);
	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

M_uintptr M_SPEC M_Export(M_CONN *conn, char *adminpwd, char *filename)
{
	M_uintptr identifier;

	if ((filename == NULL) || (adminpwd == NULL)) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_EXPORT);
	M_TransParam(conn, identifier, MC_USERNAME, "MCVEADMIN");
	M_TransParam(conn, identifier, MC_PASSWORD, adminpwd);
	if_chkvar(filename) M_TransParam(conn, identifier, MC_FILE, filename);
	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

M_uintptr M_SPEC M_Import(M_CONN *conn, char *adminpwd, char *filename)
{
	M_uintptr identifier;

	if ((filename == NULL) || (adminpwd == NULL)) {
		return M_ERROR;
	}

	identifier = M_TransNew(conn);
	M_TransParam(conn, identifier, MC_TRANTYPE, MC_TRAN_IMPORT);
	M_TransParam(conn, identifier, MC_USERNAME, "MCVEADMIN");
	M_TransParam(conn, identifier, MC_PASSWORD, adminpwd);
	if_chkvar(filename) M_TransParam(conn, identifier, MC_FILE, filename);
	if (!M_TransSend(conn, identifier)) {
		return -1;
	}

	return identifier;
}

/* LEGACY USER FUNCTIONS */

struct M_UserSetup_st {
	char *proc;
	char *user;
	char *pwd;
	char *indcode;
	char *merchid;
	char *bankid;
	char *termid;
	char *clientnum;
	char *storeid;
	char *agentid;
	char *chainid;
	char *zipcode;
	char *timezone;
	char *merchcat;
	char *mername;
	char *merchloc;
	char *statecode;
	char *servicephone;
};

void M_SPEC M_InitUserSetup(M_UserSetup *usersetup)
{
	*usersetup = Ml_malloc(sizeof(**usersetup));
	memset(*usersetup, 0, sizeof(**usersetup));
}

int M_SPEC M_AddUserArg(M_UserSetup *usersetup, int argtype, char *argval)
{
	char **field;

	if (argtype == M_PROC)
		field = &(*usersetup)->proc;
	else if (argtype == M_USER)
		field = &(*usersetup)->user;
	else if (argtype == M_PWD)
		field = &(*usersetup)->pwd;
	else if (argtype == M_INDCODE)
		field = &(*usersetup)->indcode;
	else if (argtype == M_MERCHID)
		field = &(*usersetup)->merchid;
	else if (argtype == M_BANKID)
		field = &(*usersetup)->bankid;
	else if (argtype == M_TERMID)
		field = &(*usersetup)->termid;
	else if (argtype == M_CLIENTNUM)
		field = &(*usersetup)->clientnum;
	else if (argtype == M_STOREID)
		field = &(*usersetup)->storeid;
	else if (argtype == M_AGENTID)
		field = &(*usersetup)->agentid;
	else if (argtype == M_CHAINID)
		field = &(*usersetup)->chainid;
	else if (argtype == M_ZIPCODE)
		field = &(*usersetup)->zipcode;
	else if (argtype == M_TIMEZONE)
		field = &(*usersetup)->timezone;
	else if (argtype == M_MERCHCAT)
		field = &(*usersetup)->merchcat;
	else if (argtype == M_MERNAME)
		field = &(*usersetup)->mername;
	else if (argtype == M_MERCHLOC)
		field = &(*usersetup)->merchloc;
	else if (argtype == M_STATECODE)
		field = &(*usersetup)->statecode;
	else if (argtype == M_SERVICEPHONE)
		field = &(*usersetup)->servicephone;
	else
		return 0;

	*field = Ml_malloc(strlen (argval) + 1);
	strcpy(*field, argval);
	return 1;
}


void M_SPEC M_DeleteUserSetup(M_UserSetup *usersetup)
{
	Ml_free((*usersetup)->proc);
	Ml_free((*usersetup)->user);
	Ml_free((*usersetup)->pwd);
	Ml_free((*usersetup)->indcode);
	Ml_free((*usersetup)->merchid);
	Ml_free((*usersetup)->bankid);
	Ml_free((*usersetup)->termid);
	Ml_free((*usersetup)->clientnum);
	Ml_free((*usersetup)->storeid);
	Ml_free((*usersetup)->agentid);
	Ml_free((*usersetup)->chainid);
	Ml_free((*usersetup)->zipcode);
	Ml_free((*usersetup)->timezone);
	Ml_free((*usersetup)->merchcat);
	Ml_free((*usersetup)->mername);
	Ml_free((*usersetup)->merchloc);
	Ml_free((*usersetup)->statecode);
	Ml_free((*usersetup)->servicephone);
	Ml_free(*usersetup);
}

static void M_AddEditUserSetParams(M_CONN *myconn, M_uintptr identifier, M_UserSetup *usersetup)
{
	if ((*usersetup)->proc != NULL)
		M_TransParam(myconn, identifier, MC_USER_PROC, (*usersetup)->proc);

	if ((*usersetup)->user != NULL)
		M_TransParam(myconn, identifier, MC_USER_USER, (*usersetup)->user);

	if ((*usersetup)->pwd != NULL)
		M_TransParam(myconn, identifier, MC_USER_PWD, (*usersetup)->pwd);

	if ((*usersetup)->indcode != NULL)
		M_TransParam(myconn, identifier, MC_USER_INDCODE, (*usersetup)->indcode);

	if ((*usersetup)->merchid != NULL)
		M_TransParam(myconn, identifier, MC_USER_MERCHID, (*usersetup)->merchid);

	if ((*usersetup)->bankid != NULL)
		M_TransParam(myconn, identifier, MC_USER_BANKID, (*usersetup)->bankid);

	if ((*usersetup)->termid != NULL)
		M_TransParam(myconn, identifier, MC_USER_TERMID, (*usersetup)->termid);

	if ((*usersetup)->clientnum != NULL)
		M_TransParam(myconn, identifier, MC_USER_CLIENTNUM, (*usersetup)->clientnum);

	if ((*usersetup)->storeid != NULL)
		M_TransParam(myconn, identifier, MC_USER_STOREID, (*usersetup)->storeid);

	if ((*usersetup)->agentid != NULL)
		M_TransParam (myconn, identifier, MC_USER_AGENTID, (*usersetup)->agentid);

	if ((*usersetup)->chainid != NULL)
		M_TransParam (myconn, identifier, MC_USER_CHAINID, (*usersetup)->chainid);

	if ((*usersetup)->zipcode != NULL)
		M_TransParam (myconn, identifier, MC_USER_ZIPCODE, (*usersetup)->zipcode);

	if ((*usersetup)->timezone != NULL)
		M_TransParam (myconn, identifier, MC_USER_TIMEZONE, (*usersetup)->timezone);

	if ((*usersetup)->merchcat != NULL)
		M_TransParam (myconn, identifier, MC_USER_MERCHCAT, (*usersetup)->merchcat);

	if ((*usersetup)->mername != NULL)
		M_TransParam (myconn, identifier, MC_USER_MERNAME, (*usersetup)->mername);

	if ((*usersetup)->merchloc != NULL)
		M_TransParam (myconn, identifier, MC_USER_MERCHLOC, (*usersetup)->merchloc);

	if ((*usersetup)->statecode != NULL)
		M_TransParam (myconn, identifier, MC_USER_STATECODE, (*usersetup)->statecode);

	if ((*usersetup)->servicephone != NULL)
		M_TransParam (myconn, identifier, MC_USER_PHONE, (*usersetup)->servicephone);
}

M_uintptr M_SPEC M_AddUser(M_CONN * myconn, char *adminpwd, M_UserSetup *usersetup)
{
	M_uintptr identifier;

	identifier = M_TransNew (myconn);
	M_TransParam(myconn, identifier, MC_TRANTYPE, MC_TRAN_ADDUSER);
	M_TransParam(myconn, identifier, MC_USERNAME, "MCVEADMIN");
	M_TransParam(myconn, identifier, MC_PASSWORD, adminpwd);

	M_AddEditUserSetParams(myconn, identifier, usersetup);

	if (!M_TransSend (myconn, identifier))
		return -1;
	return identifier;
}

char * M_SPEC M_GetUserArg(M_UserSetup *usersetup, int argtype)
{
	char *field;

	if (argtype == M_PROC)
		field = (*usersetup)->proc;
	else if (argtype == M_USER)
		field = (*usersetup)->user;
	else if (argtype == M_PWD)
		field = (*usersetup)->pwd;
	else if (argtype == M_INDCODE)
		field = (*usersetup)->indcode;
	else if (argtype == M_MERCHID)
		field = (*usersetup)->merchid;
	else if (argtype == M_BANKID)
		field = (*usersetup)->bankid;
	else if (argtype == M_TERMID)
		field = (*usersetup)->termid;
	else if (argtype == M_CLIENTNUM)
		field = (*usersetup)->clientnum;
	else if (argtype == M_STOREID)
		field = (*usersetup)->storeid;
	else if (argtype == M_AGENTID)
		field = (*usersetup)->agentid;
	else if (argtype == M_CHAINID)
		field = (*usersetup)->chainid;
	else if (argtype == M_ZIPCODE)
		field = (*usersetup)->zipcode;
	else if (argtype == M_TIMEZONE)
		field = (*usersetup)->timezone;
	else if (argtype == M_MERCHCAT)
		field = (*usersetup)->merchcat;
	else if (argtype == M_MERNAME)
		field = (*usersetup)->mername;
	else if (argtype == M_MERCHLOC)
		field = (*usersetup)->merchloc;
	else if (argtype == M_STATECODE)
		field = (*usersetup)->statecode;
	else if (argtype == M_SERVICEPHONE)
		field = (*usersetup)->servicephone;
	else
		return NULL;
	return field;
}

M_uintptr M_SPEC M_EditUser(M_CONN * myconn, char *adminpwd, M_UserSetup *usersetup)
{
	M_uintptr identifier;

	identifier = M_TransNew (myconn);
	M_TransParam(myconn, identifier, MC_TRANTYPE, MC_TRAN_EDITUSER);
	M_TransParam(myconn, identifier, MC_USERNAME, "MCVEADMIN");
	M_TransParam(myconn, identifier, MC_PASSWORD, adminpwd);

	M_AddEditUserSetParams(myconn, identifier, usersetup);

	if (!M_TransSend (myconn, identifier))
		return -1;
	return identifier;
}

const char * M_SPEC M_GetUserParam(M_CONN * myconn, M_uintptr identifier, int key)
{
	const char *str_key = NULL;
	
	if (!Ml_verify_trans_in_queue(myconn, identifier)) return(NULL);
	
	str_key = M_GetUserArgString(key);
	if (str_key == NULL)
		return NULL;
	return M_ResponseParam(myconn, identifier, str_key);
}

#endif /* ifndef LIBMONETRA_SPEC_STDCALL */
