#ifndef __MAIN_H__
#define __MAIN_H__

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include "monetra.h"

/* Standard includes ... all platforms */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <stdarg.h>

#ifdef HAVE_STRINGS_H
#  include <strings.h>
#endif

#ifdef HAVE_SYS_SELECT_H
#  include <sys/select.h>
#endif

/* Win32 includes */
#ifdef _WIN32
#  include <io.h>
#  include <sys/timeb.h>
#  include <winsock2.h>
#  include <windows.h>
#else /* !Win32 */
/* UNIX/POSIX includes */
#  include <unistd.h>
#  include <sys/socket.h>
#  include <netinet/in.h>
#  include <arpa/inet.h>
#  include <netdb.h>
#  include <sys/time.h>
#  include <dirent.h>
#endif /* Win32 */

#include "csv.h"

typedef struct M_TRANS {
	char *key;
	char *value;
} M_TRANS;

typedef struct M_QUEUE {
	M_uint64 identifier;
	int status;

	/* Store outgoing transaction fields -- key/value pairs */
	int transaction_fields;
	M_TRANS *transaction;

	int resp_fields;
	M_TRANS *parsed_resp;
	/* Comma Delimited response storage */
	char *response;
	int iscommadelimited;
	/* storage */
	Ml_csv_data *csv;

	void *next;
	void *prev;
} M_QUEUE;

#include "utils.h"
#include "misc.h"
#include "locking.h"
#include "sockets.h"
#include "lm_strings.h"
#include "snprintf.h"
#include "proxy.h"
#include "mstdlib.h"
#include "buf.h"

#ifdef _WIN32
#  define ioctl(a,b,c) ioctlsocket(a,b,(u_long *)c)
#  ifndef _WIN32_WCE
#    ifdef _WIN64
#      define fstat(a,b) _fstat64(a,b)
#      define stat _stat64
#    else
#      define fstat(a,b) _fstat(a,b)
#      define stat _stat
#    endif
#    define open(a,b) _open(a,b)
#    define read(a,b,c) _read(a,b,c)
#    define write(a,b,c) _write(a,b,c)
#    define close(a) _close(a)
#    define strdup(a) _strdup(a)
#  endif /* !_WIN32_WCE */
#endif /* ifdef _WIN32 */

#define M_SOCKETS 2
#define M_SSL 3

#define M_DISCONNECTED 0
#define M_CONNECTED 1

typedef struct _M_CONN {
	int conn_method; /* Drop File, Sockets, SSL */
	char location[251]; /* Either Drop-File directory, or hostname */
	unsigned short port; /* For IP and SSL, a port number */
	int ptr; /* For IP and SSL, the pointer to an open connection */

	Ml_buf_t inbuf;
	int last_parse_attempt_len; /* Optimization for large response queries */

	Ml_buf_t outbuf;

	int status;
	int do_debug;

	long timeout;
	int blocking;
	int verifyconn;
	int verifyssl;
	int validate_identifier;
	int max_conn_time;
	char *error_text;

#ifdef ENABLE_SSL
	void *ssl;
	void *server_cert;
	void *client_ctx;
	char ca_location[255];
	char ca_dir[255];
	char sslkeyfile[255];
	char sslcertfile[255];

#endif
	/* int CA_verify; */
	/* char *CA_cert; */  /* To store an SSL certificate */
	/* char *CA_key; */ /* To store an SSL key */
	long outstanding_auths;
	long queue_length;
	M_QUEUE *queue;

	M_Register_Mutex mutexreg;
	M_Mutex_Lock mutexlock;
	M_Mutex_Unlock mutexunlock;
	M_Unregister_Mutex mutexunreg;
	M_ThreadID threadid;

	void *mutex;
	proxy_config_st proxy;

	M_uint64 last_identifier;
} _M_CONN;

#define IP_READ_SIZE 8192

#endif
