#include "main.h"
#include "monetra.h"
#include "mstring.h"
#include "mstdlib.h"

/* Replacement for sleep() and usleep() */
int M_SPEC M_uwait(unsigned long length)
{
#ifdef WIN32
	Sleep(length / 1000);
#else
	fd_set readfs;
	struct timeval timeout;

	timeout.tv_sec = length / (unsigned long)1000000;
	timeout.tv_usec = (length % (unsigned long)1000000);
	FD_ZERO(&readfs);
	select(0, &readfs, NULL, NULL, &timeout);
#endif /* ifdef WIN32 */
	return 1;
}

void M_Set_Conn_Error(M_CONN *conn, const char *error)
{
	if ((error != NULL) && strlen(error)) {
		if ((*conn)->error_text != NULL) {
			Ml_free((*conn)->error_text);
		}

		(*conn)->error_text = NULL;
		(*conn)->error_text = Ml_strdup(error);
	}
}

/* NULL on failure, allocated buffer on success */
char *M_readfile(const char *path, size_t *len)
{
	int         fd;
	struct stat sb;
	char       *ret;
#ifndef O_BINARY
#  define O_BINARY 0
#endif
#ifdef _WIN32_WCE
	fd = open(path, O_RDONLY | O_BINARY, 0);
#else
	fd = open(path, O_RDONLY | O_BINARY);
#endif
	if (fd == -1)
		return NULL;

	if (fstat(fd, &sb) != 0) {
		close(fd);
		return NULL;
	}

	ret = Ml_malloc(sb.st_size + 1);
	if (read(fd, ret, sb.st_size) != sb.st_size) {
		close(fd);
		Ml_free(ret);
		return NULL;
	}
	close(fd);

	*len      = sb.st_size;
	ret[*len] = 0;
	return ret;
}
