#include "main.h"
#include "monetra.h"
#include <stdio.h>
#ifdef HAVE_STRINGS_H
#  include <strings.h>
#endif
#include <string.h>
#ifndef _WIN32
#  include <dirent.h>
#  include <unistd.h>
#else
#  include <io.h>
#endif
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "mstdlib.h"
#include "snprintf.h"
#include "mstring.h"
#include "mlfile.h"

#ifdef _WIN32
#  define Ml_PATHSEP '\\'
#else
#  define Ml_PATHSEP '/'
#endif

static int Ml_dirlist_scan(char ***filenames, int *num_files, const char *directory);

static void Ml_dirlist_add(char ***filenames_out, int *num_filenames_out, const char *dir, const char *filename)
{
	char *temp = NULL;
	size_t len;
	struct stat myst;

	len = strlen(dir);
	if (dir[len-1] == '/' || dir[len-1] == '\\')
		Ml_asprintf(&temp, "%s%s", dir, filename);
	else
		Ml_asprintf(&temp, "%s%c%s", dir, Ml_PATHSEP, filename);
	
	if (stat(temp, &myst) != 0) {
		Ml_free(temp);
		return;
	}

	if (myst.st_mode & S_IFDIR) {
		/* If a directory, and not . or .., recurse */
		if (!strcmp(filename, ".") == 0 && !strcmp(filename, "..") == 0) {
			Ml_dirlist_scan(filenames_out, num_filenames_out, temp);
		}
		Ml_free(temp);
		return;
	} else if (!(myst.st_mode & S_IFREG)) {
		/* If not a regular file, we don't need it */
		Ml_free(temp);
		return;
	}
	
	if (((*num_filenames_out) % 25) == 0) {
		(*filenames_out) = Ml_realloc((*filenames_out), ((*num_filenames_out)+25) * sizeof(**filenames_out));
	}
	
	(*filenames_out)[(*num_filenames_out)] = temp;
	(*num_filenames_out)++;
}

static int Ml_dirlist_scan(char ***filenames, int *num_files, const char *directory)
{
#ifdef _WIN32
	struct _finddata_t c_file;
	long hFile;
	char find[1024];

	M_strlcpy(find, directory, sizeof(find));
	if (directory[strlen(find)-1] != '\\')
		M_strlcat(find, "\\", sizeof(find));
	M_strlcat(find, "*.*", sizeof(find));
	
	if ((hFile = _findfirst(find, &c_file)) == -1L) return(0);
	
	do {
		Ml_dirlist_add(filenames, num_files, directory, c_file.name);
	} while (_findnext(hFile, &c_file) == 0);

	_findclose(hFile);
#else
	struct dirent *namelist;
	DIR *dir;

	dir=opendir(directory);
	if (dir == NULL) return(0);
	rewinddir(dir);
	while ((namelist=readdir(dir)) != NULL) {
		Ml_dirlist_add(filenames, num_files, directory, namelist->d_name);
	}
	closedir(dir);
#endif
	return(1);
}

int Ml_dirlist(char ***filenames, const char *directory)
{
	int num_files=0;
	*filenames = NULL;
	Ml_dirlist_scan(filenames, &num_files, directory);
	return num_files;
}

int Ml_free_dirlist(char **filenames, int num)
{
	int i;
	for (i=0; i<num; i++)
		Ml_free(filenames[i]);
	Ml_free(filenames);
	return(1);
}
