#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include "mstdlib.h"

#define M_min(a,b) ((a) <= (b) ? (a) : (b))
#define SIZEOF_SIZE_T sizeof(size_t)
#define MAX_PTR_SIZE 8

static void M_zero(void *ptr, size_t size);

void *Ml_malloc(size_t size)
{
	void *ret;

	if (size == 0) {
		return NULL;
	}

	ret = malloc(size + MAX_PTR_SIZE);
	if (ret == NULL) {
		return NULL;
	}

	memcpy(ret, &size, SIZEOF_SIZE_T);

	return (char *)ret + MAX_PTR_SIZE;
}

void *Ml_realloc(void *ptr, size_t size)
{
	void *ret;
	size_t oldsize;

	if (size == 0) {
		Ml_free(ptr);
		return NULL;
	}

	if (ptr == NULL) {
		return Ml_malloc(size);
	}

	ret = Ml_malloc(size);
	if (ret == NULL) {
		return NULL;
	}

	memcpy(&oldsize, (char *)ptr - MAX_PTR_SIZE, SIZEOF_SIZE_T);
	memcpy(ret, ptr, M_min(size, oldsize));

	Ml_free(ptr);

	return ret;
}

void Ml_free(void *ptr)
{
	size_t intsize;

	if (ptr == NULL) {
		return;
	}

	memcpy(&intsize, (char *)ptr - MAX_PTR_SIZE, SIZEOF_SIZE_T);
	M_zero(ptr, intsize);

	free((char *)ptr - MAX_PTR_SIZE);
}

/* 'memset' gets optimized away sometimes; this function shouldn't.
 * Inspired by safe_memset from Dovecot.
 */
static void M_zero(void *ptr, size_t size)
{
	volatile unsigned char *p = ptr;

	while (size--) {
		*p = (unsigned char)0;
		p++;
	}
}
