#ifndef __PROXY_H__
#define __PROXY_H__

#ifndef _WIN32
#    include <sys/socket.h>
#    include <netinet/in.h>
#    include <arpa/inet.h>
#else
#    include <winsock2.h>
#    include <windows.h>
#endif

#ifndef HAVE_SOCKLEN_T
#    define HAVE_SOCKLEN_T
typedef int socklen_t;
#endif

#include "http.h"

typedef struct no_proxy_st {
	/* If NULL, match off net/mask.
	 * if not NULL, match as suffix to hostname */
	char *host_suffix;

#if !defined (HAVE_IN_ADDR_T)
	unsigned long net;
	unsigned long mask;
#else
	in_addr_t net;
	in_addr_t mask;
#endif
} no_proxy_st;

#define PROXY_TYPE_NONE 0         /* DO NOT PROXY */
#define PROXY_TYPE_HTTP 1         /* HTTP PROXY (RFC2616) */
#define PROXY_TYPE_SOCKS5 2       /* SOCKS5 PROXY */

typedef struct proxy_config_st {
	int proxy_type;

	char *proxy_host;
	unsigned short proxy_port;
	int proxy_timeout; /* Timeout for connection to proxy */

	char *authuser;
	char *authpass;

	no_proxy_st *noproxy;
	int num_noproxy;
} proxy_config_st;

int LIBMONETRA_proxy_check_noproxy(proxy_config_st *config, const char *host);
int LIBMONETRA_proxy_tunnel_check(proxy_config_st *config, const char *host, unsigned short port);
int LIBMONETRA_proxy_tunnel_connect(proxy_config_st *config, const char *host, unsigned short port, char *error, int errlen);

/* Config */
int LIBMONETRA_proxy_config_init(proxy_config_st *config);
int LIBMONETRA_proxy_config_host(proxy_config_st *config, const char *host, unsigned short port, int proxy_type);
int LIBMONETRA_proxy_config_timeout(proxy_config_st *config, int timeout);
int LIBMONETRA_proxy_config_user(proxy_config_st *config, const char *username, const char *password);
int LIBMONETRA_proxy_config_noproxy(proxy_config_st *config, const char *noproxy, char *error, int errlen);
int LIBMONETRA_proxy_config_free(proxy_config_st *config);

#define PROXY_AUTH_NONE 0
#define PROXY_AUTH_BASIC 1
#define PROXY_AUTH_DIGEST 2

typedef struct proxy_http_auth_st {
	int type;
	char *realm;
	char *nonce;
	char *qop;
	char *opaque;
	char *algorithm;
	char *domain;
} proxy_http_auth_st;

/* Helpers for external libraries.  Typically shouldn't be used directly */
int LIBMONETRA_proxy_http_parse_authenticate(M_http_t http, proxy_http_auth_st *auth);
int LIBMONETRA_proxy_http_clear_authenticate(proxy_http_auth_st *auth);
char *LIBMONETRA_proxy_authorization_response_header(proxy_http_auth_st *auth, const char *username, const char *password, const char *http_method);
int LIBMONETRA_connect_nonblock(int sockfd, const struct sockaddr *serv_addr, socklen_t addrlen, int maxtime, char *error, int errlen);
int LIBMONETRA_checkread(int fd, long timeout_s);
int LIBMONETRA_ghbn(const char *host, char **addr, char *error, int errlen);

#endif
